//RGB light V4
#include <EEPROM.h>
#include "math.h"

#define addR 0  //EEPROM Address for Red
#define addG 1  //EEPROM Address for Green
#define addB 2  //EEPROM Address for Blue

#define RED 3   //pin controlling the RED LED
#define GREEN 5 //pin controlling the GREEN LED
#define BLUE 6  //pin controlling the BLUE LED

#define buzzPin 13    //Pin controlling the buzzer
#define touchPin 2    //touch input pin

//restroring the last values of RGB from EEPROM
uint8_t valR=EEPROM.read(addR),valG=EEPROM.read(addG),valB=EEPROM.read(addB); 

//function to set RGB from 0-360 input, similar to HSV to RGB
void setRGBvalues(int H){
    H = H%360;
    float X = 1-abs(fmod(H/60.0, 2)-1);
    float r,g,b;
    if(H >= 0 && H < 60){
        r = 1,g = X,b = 0;
    }
    else if(H >= 60 && H < 120){
        r = X,g = 1,b = 0;
    }
    else if(H >= 120 && H < 180){
        r = 0,g = 1,b = X;
    }
    else if(H >= 180 && H < 240){
        r = 0,g = X,b = 1;
    }
    else if(H >= 240 && H < 300){
        r = X,g = 0,b = 1;
    }
    else{
        r = 1,g = 0,b = X;
    }

    valR = r*255;
    valG = g*255;
    valB = b*255;

    analogWrite(RED,valR);
    analogWrite(GREEN,valG);
    analogWrite(BLUE,valB);
}


//initatiang the timer1 of atmega328p for keeping the 4-7-8 sec timing
void timer1Init(){
  TCCR1A = 0;
  TCCR1B = 0;                     //stop existing Timer
  TCNT1 = 0;                      // Clear timer count
  TIFR1 = 255;                    // Clear  pending interrupts

  OCR1A = 250; 

  //setting the mode to CTC by WGM 3:0 = 0100
  TCCR1A &= ~((1<<WGM11)|(1<<WGM10));
  TCCR1B |= (1<<WGM12);
  TCCR1B &= ~(1<<WGM13);

  //setting the interrupt register
  TIMSK1 |= (1<<OCIE1A);

  //turing on the global interrups
  sei();

  //starting the timer1 by setting prescaller to 1024 i.e. CS1 (2:0) = 101
  TCCR1B &= ~(1<<CS11);
  TCCR1B |= (1<<CS12)|(1<<CS10);
}

volatile float cR=0.00, cG=0.00, cB=0.00;
volatile float fR = valR/250.0, fG = valG/250.0, fB = valB/250.0;

volatile long int touchTime=0, releaseTime=0;  //will store the time when the button is touched & relesed
volatile boolean isPress=0;         //will store wheather the touch button is pressed or not
volatile boolean play=1;            //1 for playing 0 for pausing
volatile boolean mode=0;            //1 for color setting mode, 0 for normal 4-7-8 mode
volatile int colorHSV = 0;          //will store the HSV's Hue number to change the color for color setting mode
volatile uint16_t inPlayCounter= 0; //it is the counter which will tick inbetween 4-7-8
volatile uint8_t inPlayMode=0;      //0 for 4s period, 1 for 7s perios & 2 for 8s period


//this interrupt will handle the contions for color setting mode and fading in/out of choosen color
ISR(TIMER1_COMPA_vect){
      if(mode && !isPress){ //this is the mode for setting the color
    setRGBvalues(colorHSV);
    colorHSV++;
    if(colorHSV==360) colorHSV=0;
  }else if(!mode){        //glow & fade mode
      inPlayCounter++;
      switch(inPlayMode){ //switch case to determine to glow or fade out or pause 
          case 0:{    //for glowing in
            cR+=fR;
            cG+=fG;
            cB+=fB;
            if(inPlayCounter==250){ //for 4 sec 
                cR=valR;
                cG=valG;
                cB=valB;
                inPlayCounter=0;
                inPlayMode=1;
              }
            break;}
          case 1:{    //pausing for 7 seconds
            //no change in cR in this section
            if(inPlayCounter==438){ //for 7 sec we will count up to 438 as each timer interrupt occurs at 4ms
                inPlayCounter=0;
                inPlayMode=2;
              }
            break;}
          case 2:{    //for fading out
            cR -=fR/2;
            cG -=fG/2;
            cB -=fB/2;
            if(inPlayCounter==500){ //for 8 sec we will count up to 500 as each timer interrupt occurs at 4ms
                cR=0;
                cG=0;
                cB=0;
                inPlayCounter=0;
                inPlayMode=0;
              }
            break;}
        }
     analogWrite(RED,cR);
     analogWrite(GREEN,cG);
     analogWrite(BLUE,cB);
  }
}

//function which will be called when touch button is pressed
void interruptFunction(){
  if(digitalRead(touchPin)){
     touchTime = millis();  //record the tiem when digital pin goes high
     isPress=1;
     return;
  }
  else
     releaseTime = millis();  //record the tiem when digital pin goes low
}

void setup() {
  // put your setup code here, to run once:
  pinMode(RED,OUTPUT);
  pinMode(GREEN,OUTPUT);
  pinMode(BLUE,OUTPUT);
  pinMode(touchPin,INPUT);
  pinMode(buzzPin,OUTPUT);
  Serial.begin(9600);
  Serial.println("EEPROM Restorerd Values");
  Serial.print("Red : ");
  Serial.println(valR);
  Serial.print("Green : ");
  Serial.println(valG);
  Serial.print("Blue : ");
  Serial.println(valB);

  //initiate the timer1 
  timer1Init();
  attachInterrupt(digitalPinToInterrupt(touchPin),interruptFunction,CHANGE);
}

//variable to control the on time of buzzer for different modes
signed long int buzzTime = 0;
boolean buzzOn = 0;

//del input will determine how long will the buzzer will buzz
void buzz(int del){
    buzzOn = 1;
    buzzTime = millis()+del;
    digitalWrite(buzzPin,1);
    Serial.println(buzzTime);
  }

void loop(){
   long int elapsedTime = releaseTime-touchTime;
  if(isPress){
      if( elapsedTime > 100 && elapsedTime < 1000 )  // short touch is over
      { 
        buzz(200);
        digitalWrite(buzzPin,1);
        Serial.println("Short Press");
        play = !play;
        if(play){
                timer1Init();
          }else{
                TCCR1B = 0;
          }
        isPress=0;
      }
      else if(millis()-touchTime > 1000 && digitalRead(touchPin) )  //long press
      {
        buzz(500);
        Serial.println("Long Press");
        mode=!mode;//FIRST CHANGE THE MODE
        if(!mode){  //THE COLOR PLAYING MODE
          inPlayMode = 0;
          inPlayCounter=0;
          cR=0;cG=0;cB=0;
          fR = valR/250.0;
          fG = valG/250.0;
          fB = valB/250.0;
          Serial.println("eeprom Updated");
          EEPROM.update(addR,valR);
          EEPROM.update(addG,valG);
          EEPROM.update(addB,valB);
        }
        isPress=0;
      }
      else if(elapsedTime<200 && digitalRead(touchPin)==0){
        isPress=0;
      }

  }

  Serial.print("buzzTime : ");
  Serial.print(signed(buzzTime-millis()));
  if(buzzOn && (signed(buzzTime-millis())<0))
  {
      digitalWrite(buzzPin,0);
      buzzOn = 0;
  }
}
